function   [mas,wmas,fas,wfas,w]=as(mu,g,fre,fco,wmin,wmax,N)

% Per una funzione di trasferimento G(s) in forma fattorizzata (costanti di
% tempo), la funzione:
% 	[mas,wmas,fas,wfas,w]=as(muG,gG,freG,fcoG,w1,w2,N);
% calcola i diagrammi di Bode asintotici. In particolare:
%
% muG,gG     sono il guadagno e il tipo di G(s)
% freG,fcoG  sono matrici desunte da G(s) come segue:
%	       freG : 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	       fcoG : 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli)
% w1,w2 sono gli estremi della banda d'interesse. Se omessi, oppure se 
%  	w1=inf, w1 e w2 sono fissati automaticamente in modo da comprendere
% 	tutte le pulsazioni d'angolo di G(s).
% N	e' il numero di elementi del vettore w. In assenza, N=500.
%
% mas,wmas  danno il diagr. di Bode asint. del modulo [semilogx(wmas,mas)]
% fas,wfas  danno il diagr. di Bode asint. della fase [semilogx(wfas,fas)]
% w	e' un vettore di N valori di pulsazione, uniformemente distribuiti
%	sulla piu' piccola banda in decadi contenente [w1,w2].

% G.O.Guardabassi, 1 gennaio 1997, 19 marzo 1998

inwmm=1;
if nargin < 7, N=500; end
if nargin < 5, wmin=inf; end
if nargin < 4, fco=[]; end
if nargin < 3, fre=[]; end

[nr,nc]=size(fre);
if nc>0 
	wT=ones(1,nc)./fre(1,:);
	swT=sign(wT); 
	mwT=fre(2,:); 
else 
	wT=[]; swT=[]; mwT=[]; 
end
[nr,nc]=size(fco);
if nc>0 
	wn=fco(1,:);
	swn=sign(fco(2,:)+eps*ones(1,nc)); 
	mwn=2*fco(3,:); 
else 
	wn=[]; swn=[]; mwn=[]; 
end
wx=abs([wT wn]);
sig=[swT swn];
mol=[mwT mwn];

if wmin==inf
	if length(wx)>0
		wmin=min(wx)/3;wmax=max(wx)*3;
	else
		if g==0 
			wmin=.01; wmax=100; 
		else
			wo=mu^(1/g);
			wmin=wo/10; wmax=wo*10;
		end
	end
end

d1=floor(log10(wmin));
d2=ceil(log10(wmax));
w=logspace(d1,d2,N);

[wy ksort]=sort(wx);
moly=mol(ksort);
sigy=sig(ksort);
wy=min(w(N),max(w(1),wy));			

pend=cumsum([-g moly]);
mosi=mol.*sig;
mosy=mosi(ksort);
pens=cumsum([-g mosy]);

wmas=[w(1) wy w(N)];
mas(1)=20*log10(abs(mu/(w(1)^g)));
for i=1:1+length(wx)
	mas(i+1)=mas(i)+pend(i)*20*log10(wmas(i+1)/wmas(i));
end
wfas=[w(1) sort([wy wy]) w(N)];
fas=kron(90*pens,[1 1]);
if mu<0
	fas=fas-180*ones(size(fas));
end
