function [mdf,omegac,mdg,omegap,muRv,gRv,freRv,fcoRv]=...
   cadbode(i,muX,gX,freX,fcoX,muG,gG,freG,fcoG)

% La funzione:		cadbode(i,muX,gX,freX,fcoX,muG,gG,freG,fcoG); 
%  di ausilio nella progettazione per tentativi di un sistema di controllo mono- 
% variabile,lineare e invariante nel tempo. Le modalit d'impiego sono due:
% * In ogni caso, G(s)  la funz. di trasf. del sistema sotto controllo:
%	muG,gG     sono il guadagno e il tipo di G(s)
%	freG,fcoG  sono matrici desunte da G(s) come segue:	
%		freG:	1a riga, costanti di tempo reali (al num. o al den.)
%	       	      	2a riga, corrispondenti molteplicita' (negative se al den.)
%		fcoG:	1a riga, pulsazioni naturali relative a coppie di zeri o
%	       	      		 poli complessi coniugati
%	       	      	2a riga, corrispondenti fattori di smorzamento
%	       	      	3a riga, corrispondenti molteplicita' (negative se poli) 
% * Quindi, se si pone i=0, allora X=R e R(s)  la funz. di trasf. del controllore:		     
%	 	muR,gR,freR,fcoR specificano (come sopra) la R(s) di primo tentativo.
% * Se invece si pone i=1, allora X=L e L(s)  la funz. di trasf. d'anello desid.
%		muLo,gLo,freLo,fcoLo specificano (come sopra) la L(s).
% I tentativi possono susseguirsi sulla base dei diagrammi di Bode via via elaborati 
% dal programma. Se si pone:
%
% [mdf,omegac,mdg,omegap,muRf,gRf,freRf,fcoRf]=cadbode(i,muX,gX,freX,fcoX,muG,gG,freG,fcoG);
%
% il programma restituisce, nello spazio di lavoro. oltre al margine di fase mdf, alla 
% pulsazione critica omegac, al margine di guadagno mdg e alla pulsazione di attravers.
% del semiasse reale negativo omegap, la funzione di trasferimento Rf(s) (specificata 
% con le modalit consuete da: muRf,gRf,freRf,fciRf) dell'ultimo controllore esaminato.

% G.O.Guardabassi, 10 gennaio 1997, 19 marzo 1998, 27 febbraio 2000

run Rv

global muRv gRv freRv fcoRv

[mdf,omegac,mdg,omegap,muRv,gRv,freRv,fcoRv]=...
   cadbode1(i,muX,gX,freX,fcoX,muG,gG,freG,fcoG,muRv,gRv,freRv,fcoRv);

muRf=muRv;
gRf=gRv;
freRf=freRv;
fcoRf=fcoRv;

disp(' ')
disp(sprintf(' Margine di fase     = %5.0f  ',mdf))
disp(' ') 
disp(sprintf(' Pulsazione critica  = %0.4g  ',omegac))
disp(' ')
disp(sprintf(' Margine di guadagno = %5.1f  ',mdg))
disp(' ')
disp(sprintf(' Pulsazione a -180  = %0.4g  ',omegap))
disp(' ')
