function    [w,mR,fR,mRo,fRo]=cdrif(nuR,deR,nuRo,deRo,T)

% La funzione
%		[w,mR,fR,mRo,fRo]=cdrif(nuR,deR,nuRo,deRo,T);
%
% mostra i diagrammi di Bode, del modulo e dell'argomento, relativi alle 
% r.i.f. di R(z) (linea continua) e di R(s) (linea tratteggiata), sulla
% banda formata dalla decade in cui cade la pulsazione di Nyquist (pi/T)
% e dalle 3 decadi precedenti.
%
% nuR,deR	sono il numeratore e il denominatore di R(z)
% nuRo,deRo	sono il numeratore e il denominatore di R(s)
% T		e' il periodo di campionamento
%
% Se richiesti, vengono restituiti i valori della pulsazione w e di:
% mR=|R(exp(jwT))|, fR=Arg(R(exp(jwT))), mRo=|R(jw)|, fRo=Arg(R(jw)).

% G.O.Guardabassi, 10 gennaio 1997, 25 marzo 1998, 8 marzo 2000.

N=500;
wnyq=pi/T;
d2=ceil(log10(wnyq));
d1=d2-4;
w=logspace(d1,d2,N);

[mRo,fRo]=bode(nuRo,deRo,w);
mRodB=20*log10(mRo);

[mR,fR]=dbode(nuR,deR,T,w);
mRdB=20*log10(mR);

ew=[w(1) w(N)];
ze=[0 0];

clc
disp(' ')
disp('******************************* ATTENZIONE *********************************')
disp('* Seguiranno 3 figure  con i diagrammi di Bode del modulo e dell''argomento *')
disp('* di R e di R, sulla banda formata dalla decade in cui cade la pulsazione *')
disp('* di Nyquist e dalle 3 decadi precedenti.                                  *')
disp('*                                                                          *')
disp('* Per vedere la prima figura, o per passare alla successiva,               *')
disp('*                                                        BATTERE UN TASTO. *')
disp('****************************************************************************')
disp(' ')
pause;


figure(1),close,figure(1)
    semilogx(w,mRodB,':r',ew,ze,'-k',w,mRdB,'-k'),grid
    title('Modulo: R  , R ------')
    xlabel('w'),ylabel('dB'),pause;

figure(2),close,figure(2)
    semilogx(w,fRo,':r',ew,ze,'-k',w,fR,'-k',w(1),5,'-k'),grid
    title('Argomento: R  , R ------')
    xlabel('w'),ylabel('[]'),pause;

figure(3),close,figure(3)
    subplot(211),semilogx(w,mRodB,':r',ew,ze,'-k',w,mRdB,'-k'),grid
    title('Modulo: R  , R ------'),ylabel('dB')
    subplot(212),semilogx(w,fRo,':r',ew,ze,'-k',w,fR,'-b',w(1),5,'-k'),grid
    title('Argomento: R  , R ------'),xlabel('w'),ylabel('[]'),subplot
