function     E0=etambm(E1,M,B,G0,yo,uo)

% La funzione     E0=etambm(E1,M,B,G0,yo,uo);     calcola E0=eta(E1) quando:
% l'elemento non lineare e' un MB/2,  G0 e' il limite per s-->0 di G(s) (se
% g<0, G0=0; se g=0, G0=mu; se g>0, G0=inf; mu,g sono il guadagno e il tipo 
% di G), yo e' il riferimento e uo il "carico". In mancanza, uo=0 e yo=0.
%
% Se E1 e' un vettore positivo 1xm, allora E0 e' una matrice qxm, q>=1. Gli
% elementi della k-esima colonna di E0, diversi da NaN, sono i possibili va-
% lori di eta(E1(k)).

% G.Guardabassi, 2 gennaio 1998

if nargin < 6, uo=0; end
if nargin < 5, yo=0; end

if min(E1) <= 0
	disp(' ') 
	disp('E1 deve essere positivo. Calcolo interrotto.')
	disp(' ')
	return
end
dim=size(E1);
if dim(1) > dim(2), E1=E1'; end

N=200;
uno=ones(1,N);
E0=ones(dim)*NaN;
nr=1;
for k=1:length(E1)
	x=E1(k);
	a=abs(x-B/2);
	y=linspace(-a,a,N);
	sigma=uno*B*.5/x+y/x;
	delta=uno*B*.5/x-y/x;
	Do=M*(gg(sigma)-gg(delta))./y;
	if G0==0 
		E0k=yo;
	else
		z=uno*uo+Do.*y+(y-uno*yo)/G0;
		E0k=zeri(y,z)';
	end
	if (length(E0k) <= nr) & (length(E0k) > 0)
		E0(1:length(E0k),k)=E0k;
	elseif length(E0k) > nr
		EE=E0;
		E0=ones(length(E0k),dim(2))*NaN;
		E0(1:nr,:)=EE;
		E0(1:length(E0k),k)=E0k;		
		nr=length(E0k);
	end
end
		






