function     U0=etasat(U1,v1,v2,R0,G0,yo,dc)

% La funzione   U0=etasat(U1,v1,v2,R0,G0,yo,dc);  calcola U0=eta(U1) quando:
% l'elemento non lineare e' una saturazione [v=sat(u;v1,v2)] posta fra R(s)
% e G(s), R0 (G0) e' il limite per s-->0 di R(s) (G(s))  (se gR<0, R0=0; se
% gR=0, R0=muR; se gR>0, R0=inf),  yo e' il riferimento e dc un disturbo di 
% carico costante su v (ingresso di G). In mancanza, yo=0 e dc=0.
%
% (G0=0, R0=inf) o (G0=inf, R0=0) sono escluse (canc. nell'origine)
%
% Se U1 e' un vettore positivo 1xm, allora U0 e' una matrice qxm, q>=1. Gli
% elementi della k-esima colonna di U0, diversi da NaN, sono i possibili va-
% lori di eta(U1(k)).

% G.Guardabassi, 5 gennaio 1998


N=200;
if min(U1)<0
	disp(' ')
	disp('ATTENZIONE! Gli elementi di U1 non possono essere negativi.')
	disp('                                        Esecuzione sospesa ')
	disp(' ')
	return
end
vm=(v2+v1)/2;
delta=(v2-v1)/2;
uno=ones(1,N);
U0=[];

for k=1:length(U1)
	x=max(eps,U1(k));
	a=x+max(abs(v1),abs(v2));
	y=linspace(-a,a,N);
	gamma=y-vm*uno;
	[f1 f0p]=ff((delta*uno+gamma)/x);
	[f1 f0m]=ff((delta*uno-gamma)/x);
	No=.5*x*(f0p-f0m)./gamma;
	Do=(vm*uno+No.*gamma)./y;
	if R0*G0==0
		if R0==inf
			disp(' ')
			disp('ATTENZIONE! Cancellazione illegittima.')
			disp('                   Esecuzione sospesa')
			disp(' ')
			return
		end
		U0k=yo*R0;
	else
		z=dc*uno+Do.*y-(yo*uno-y/R0)/G0;
		U0k=zeri(y,z)';
	end
	if k==1 & isempty(U0k)==1, U0k=NaN; end
	[p,q]=size(U0);
	if length(U0k) > p 
		U0=[U0;NaN*ones(p-length(U0k),q)]; 
		U0kc=[]; 
	else
		U0kc=NaN*ones(p-length(U0k),1);
	end
	U0=[U0 [U0k;U0kc]];
end









