function [num,den]=f2nd(mu,g,fre,fco)

% La funzione      [nuG,deG]=f2nd(muG,gG,freG,fcoG);
%
% calcola numeratore (nuG) e denominatore (deG) di una funzione di trasfe-
% rimento G(s) data in forma fattorizzata (costanti di tempo):
%
% muG,gG     sono il guadagno e il tipo di G(s)
% freG,fcoG  sono matrici desunte da G(s) come segue:	
%            fre : 1a riga, costanti di tempo reali (al num. o al den.)
%                  2a riga, corrispondenti molteplicita' (negative se al 
% 		            denominatore)
%	     fco : 1a riga, pulsazioni naturali relative a coppie di zeri o
% 			    poli complessi coniugati
% 	       	   2a riga, corrispondenti fattori di smorzamento
% 	       	   3a riga, corrispondenti molteplicita' (negative se poli) 
% 			       
% In mancanza, fre=[] e fco=[].

% G.O.Guardabassi, 31 gennaio 1998, 8 febbraio 2000

if nargin < 4, fco=[]; end
if nargin < 3, fre=[]; end

if g>=0
	nu=mu;
	de=[1 zeros(1,g)];
else
	nu=mu*[1 zeros(1,-g)];
	de=1;
end

if ~isempty(fre)
   ir=find(fre(2,:)>0);
	for k=1:length(ir)
		irk=ir(k);
		for kk=1:fre(2,irk)
			nu=conv(nu,[fre(1,irk) 1]);
		end
	end
	ir=find(fre(2,:)<0);
	for k=1:length(ir)
		irk=ir(k);
		for kk=1:-fre(2,irk)
			de=conv(de,[fre(1,irk) 1]);
		end
	end
end

if ~isempty(fco)
	ir=find(fco(3,:)>0);
	for k=1:length(ir)
		irk=ir(k);
		for kk=1:fco(3,irk)
			nu=conv(nu,[1/fco(1,irk)^2 2*fco(2,irk)/fco(1,irk) 1]);
		end
	end
	ir=find(fco(3,:)<0);
	for k=1:length(ir)
		irk=ir(k);
		for kk=1:-fco(3,irk)
			de=conv(de,[1/fco(1,irk)^2 2*fco(2,irk)/fco(1,irk) 1]);
		end
	end
end

num=nu; den=de;
num=[zeros(1,length(den)-length(num)) num];
