function     [nuR,deR]=fdtpid(Kp,Ti,Td,N);

%                    [nuR,deR]=fdtpid(Kp,Ti,Td,N);     
% calcola il numeratore e il denominatore della funzione di trasferimento R(s)
% di un PID (o di un PI), dati i parametri Kp,Ti,Td e N (o Kp e Ti). 
% Se i parametri disponibili fossero Ki e Kd, anziche' Ti e Td, sostituire
% Ti con Kp/Ki e Td con Kd/Ki. In assenza: N=10.
%
% Ponendo:  [muR,gR,freR,fcoR]=fdtpid(Kp,Ti,Td,N);  si ottiene la funzione
% di trasferimento R(s) del controllore in forma fattorizzata:
%   muR,gR     guadagno e tipo di R(s)
%   freR,fcoR  matrici desunte da R(s) come segue:	
%	       freR : 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	       fcoR : 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli)

% G.Guardabassi, 2 gennaio 2000

if nargin == 3, N=10; end

if nargin > 2
	nuR=[Ti*Td*(1+1/N) Ti+Td/N 1]*Kp;
	deR=[Ti*Td/N Ti 0];
else
   nuR=Kp*[Ti 1];
   deR=[Ti 0];
end

if nargout > 2, [muR,gR,freR,fcoR]=nd2f(nuR,deR); end



