function [E,W,Em]=mdfdm2(M,B,mu,g,fre,fco,E1,wp,yo,uo)

%* La funzione mdfdmbm fornisce, in 2 figure a diversa risoluzione, il 
%  diagramma polare della r.i.f., relativo ad una funzione di trasferimento
%  G(s), e il luogo dei punti critici relativo a un elemento MB/2.
%* Se num e den sono i vettori "numeratore" e "denominatore" di G(s), allora:
% 		       [E,W]=mdfdmbm(M,B,num,den);
%  oppure se:  
%       mu,g    sono il guadagno e il tipo di G
%      fre,fco  sono matrici desunte da G come segue:	
%	        fre : 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	   	fco : 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli)
%  allora       [E,W]=mdfdmbm(M,B,mu,g,fre,fco);        visualizza la parte
%  rilevante del diagramma polare di G(jw) e restituisce i valori di E e W 
%  corrispondenti agli eventuali punti d'intersezione con il luogo dei p.c..
%* Se E1p,wp sono vettori di raffinamento locale ("attorno" a E(i),w(i)),
%		[E1,W,E0]=mdfdmbm(M,B,num,den,E1p,wp,yo,uo);
%  (oppure:   [E1,W,E0]=mdfdmbm(M,B,mu,g,fre,fco,E1p,wp,yo,uo);            )
%  visualizza (e da') la soluzione, applicando le f.d. a duplice ingresso.
%  La i-esima colonna di E0 corrisponde a E1(i). In mancanza, uo=0, yo=0.
%* L'orientamento e' indicato da un asterisco nel punto finale (se in fig.).

% G.O.Guardabassi, 2 gennaio 1998, 20 marzo 2000.

N=600;
					
if length(g) > 1
	num=mu;den=g;
	if den(length(den))==0 
		G0=inf; 
	else 
		G0=num(length(num))/den(length(den));
	end
	waz=abs(roots(num));
	wap=abs(roots(den));
	iz=find(waz>0);ip=find(wap>0);
	wa=[waz(iz);wap(ip)];
	ro=10;
	if g>0, ro=3; end		    
	wmin=min(wa)/ro;wmax=max(wa)*ro;      
	if nargin == 8, uo=wp; yo=E1; end
	if nargin == 7, uo=0; yo=E1; end
	if nargin == 6, uo=0; yo=0; end
	if nargin > 4, wp=fco; E1=fre; end
	FD=2; if nargin==4, FD=1; end

	d1=floor(log10(wmin));
	d2=ceil(log10(wmax));
	w=logspace(d1,d2,N);
else
	[num,den]=f2nd(mu,g,fre,fco);   
	if g==0,G0=mu;
		elseif g>0,G0=inf;
		elseif g<0,G0=0;
	end
	FD=2; if nargin==6, FD=1; end

	wangol=[];wnatur=[];
   if isempty(fre)==0,wangol=ones(1,length(fre(1,:)))./fre(1,:);end
   if isempty(fco)==0,wnatur=fco(1,:);end
	wx=abs([wangol wnatur]);
	wmin=min(wx)/10; wmax=max(wx)*10;
	d1=floor(log10(wmin));
	d2=ceil(log10(wmax));
	d11=d1-1;
	if g>0, d11=d1; end
	w=logspace(d11,d2+1,N);

	if nargin < 10, uo=0; end
	if nargin < 9 , yo=0; end
end

if FD==1    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% FD a semplice ingresso

[re,im]=nyquist(num,den,w); Y=-pi*B/(8*M);	
xmin=min(0,min(re)); xmax=max(0,max(re)); 
ymin=min(Y,min(im)); ymax=max(0,max(im));
LLx=xmax-xmin; LLy=ymax-ymin;
a1=(xmax+xmin)/2-.55*LLx; a2=a1+1.1*LLx;
a3=(ymax+ymin)/2-.55*LLy; a4=a3+1.1*LLy;

dLL=1.1*(LLx-LLy)/2;
if LLx > LLy
	a3=a3-dLL; a4=a4+dLL;
else
	a1=a1+dLL; a2=a2-dLL;
end

a=[a1 a2 a3 a4]; ex=[a1 a2]; ey=[a3 a4]; ze=[0 0];

pcx=[a1 0]; pcy=[1 1]*Y;

clc
disp(' ')
disp('  *************************** ATTENZIONE *************************')
disp('  * Seguiranno due rappresentazioni, a diversa risoluzione, del  *')
disp('  * diagramma polare di G(jw) e del luogo dei punti critici.     *')
disp('  * Per vedere la prima,  o per passare alla successiva,         *')
disp('  *                                            BATTERE UN TASTO. *')
disp('  ****************************************************************')
disp(' ')
pause;

yminr=Y-LLy/10; ymaxr=Y+LLy/10;
ii=find((re<=0)&(im>=yminr)&(im<=ymaxr));
m=length(ii);

if m==0
  disp(' ')
  disp('  *************************** ATTENZIONE *************************')
  disp('  * Il diagramma polare di G(jw) e il luogo dei punti critici di *')
  disp('  * N non si intersecano.  Pertanto, la seconda rappresentazione *')
  disp('  * viene omessa.                                                *')
  disp('  *                                            BATTERE UN TASTO. *')
  disp('  ****************************************************************')
  disp(' ')
  pause
  E=[];
  W=[];
  Em=[];
end

figure(1)
	 plot(re,im,'-k',ex,ze,'-k',ze,ey,'-k',re(N),im(N),'*g',pcx,pcy,'-r',...
    pcx(1),pcy(1),'*g'),axis(a);axis('square');grid,xlabel('Re'),ylabel('Im'),
  	 title('Diagramma polare della r.i.f. di G (in nero) e l.p.c. (in rosso)'),
 	 pause;axis;

if m>0
	xminr=min(re(ii)); xmaxr=max(re(ii)); 
	LLxr=xmaxr-xminr; 
	a1r=(xmaxr+xminr)/2-.55*LLxr; a2r=a1r+1.1*LLxr;
	a3r=Y-.12*LLy; a4r=Y+.12*LLy;
	dLLr=1.1*(LLxr-.24*LLy)/2;
	if LLxr > .24*LLy
		a3r=a3r-dLLr; a4r=a4r+dLLr;
	else
		a1r=a1r+dLLr; a2r=a2r-dLLr;
	end
	ar=[a1r a2r a3r a4r];exr=[a1r a2r];eyr=[a3r a4r];

	W=zeri(w(ii),im(ii)-Y*ones(m,1));
	aar=spline(w(ii),re(ii),W);
	E=.5*sqrt(ones(1,length(aar))*B^2+(aar*8*M/pi).^2);
	Em=zeros(size(E));

	figure(2)
       plot(re(ii),im(ii),'-k',exr,pcy,'-r',exr,[0 0],'-k',[0 0],...
	    eyr,'-k'),axis(ar);axis('square');grid,xlabel('Re'),ylabel('Im')
	    title('Diagramma polare della r.i.f. di G (in nero) e l.p.c. (in rosso)')
end
axis;

elseif FD==2    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  FD a duplice ingresso

[rep,imp]=nyquist(num,den,wp); 
w=linspace(wp(1),wp(length(wp)),100);
[re,im]=nyquist(num,den,w);

xmin=min(re); xmax=max(re); 
ymin=min(im); ymax=max(im);
LLx=xmax-xmin; LLy=ymax-ymin;
a1=(xmax+xmin)/2-.55*LLx; a2=a1+1.1*LLx;
a3=(ymax+ymin)/2-.55*LLy; a4=a3+1.1*LLy;
dLL=1.1*(LLx-LLy)/2;
if LLx > LLy
	a3=a3-dLL; a4=a4+dLL;
else
	a1=a1+dLL; a2=a2-dLL;
end
a=[a1 a2 a3 a4];

lw=length(w);
figure(1)
	 plot(re,im,'-k'),axis(a);axis('square');grid
	 title('Diagramma polare della r.i.f. di G (in nero) e l.p.c. (in rosso)')
    xlabel('Re'),ylabel('Im'),hold,plot(rep,imp,'+k')
    plot([0 0],a(3:4),'-k',a(1:2),[0 0],'-k',re(lw),im(lw),'*g')

Ez=etambm(E1,M,B,G0,yo,uo);
dEz=size(Ez);
E=[];W=[];

for k=1:dEz(1)
	E0=Ez(k,:);
	ii=find(isnan(E0)==0);
	ll=length(ii);
	if ll > 1
		E0k=E0(ii);
		E1k=E1(ii);
		slack=ones(size(E1k))./E1k;
		sigmak=0.5*B*slack+E0k./E1k;
		deltak=0.5*B*slack-E0k./E1k;
		[gggg g1ks]=gg(sigmak);
		[gggg g1kd]=gg(deltak);
		D1k=2*M*slack.*(g1ks+g1kd-j*B*slack)/pi;
		H1k=-D1k.^(-1);
		rk=real(H1k);
      ik=imag(H1k);
      
      plot(rk,ik,'-r',rk,ik,'+r',rk(ll),ik(ll),'*g')
      
		[dHGk wk]=dhg(H1k,re+j*im,w);
		Ek=zeri(E1k,dHGk);
		for i=1:length(Ek)
		  ih=find(E1k>Ek(i));
		  h=ih(1);
		  Wk=wk(h)-(wk(h)-wk(h-1))*(E1k(h)-Ek(i))/(E1k(h)-E1k(h-1));
		  E=[E Ek(i)];
		  W=[W Wk];
		end
	end
end
Em=etambm(E,M,B,G0,yo,uo); 
axis;hold;

elseif nargin < 4 | nargin == 5 %%%%%%%%%%%%%%%%%%%%%%%%%%%   Dati incongrui
	disp(' ')
	disp('   ATTENZIONE!')
	disp('   Ci sono troppo pochi argomenti.') 
	disp('                          Esecuzione sospesa.')
elseif nargin >10 
	disp(' ')
	disp('   ATTENZIONE!')
	disp('   Ci sono troppi argomenti.') 
	disp('                          Esecuzione sospesa.')
end
