function [E,W,Em]=mdfdsat(v1,v2,muR,gR,freR,fcoR,muG,gG,freG,fcoG,yo,dc)

%* La funzione mdfdsat fornisce, in 2 figure a diversa risoluzione, il 
%  diagramma polare della r.i.f., relativo a R(s)*G(s), e il luogo dei punti 
%  critici relativo a un elemento v(u)=Sat(u;v1,v2), posto fra R e G.
%* Se nuR,deR,nuG,deG sono i vettori "numeratore" e "denominatore" di R e G,
%  allora:	    [U,W]=mdfdsat(v1,v2,nuR,deR,nuG,deG);
%  oppure se:  
% [  muR,gR,muG,gG        sono il guadagno e il tipo di R e di G
%    freR,fcoR,freG,fcoG  sono matrici desunte da R e da G come segue:	
%	   fre : 1a riga, costanti di tempo reali (al num. o al den.)
%	         2a riga, corrispondenti molteplicita' (negative se al den.) 
%	   fco : 1a riga, puls. nat. di coppie di zeri (poli) compl. coniug.
%	       	 2a riga, corrispondenti fattori di smorzamento
%	       	 3a riga, corrispondenti molteplicita' (negative se poli)
%  allora:     [U,W]=mdfdsat(v1,v2,muR,gR,freR,fcoR,muG,gG,freG,fcoG);     ] 
%  visualizza il diagr. pol. di R(jw)*G(jw) e il luogo dei p. c.; inoltre, 
%  restituisce i valori di U e W corrispondenti agli eventuali punti d'in-
%  tersezione (l'i-esimo elemento di U corrisp. all'i-esimo elemento di W). 
%* Se dc e' un disturbo di carico su v (ingresso di G), e yo un rif. cost.,
%     [U1,W,U0]=mdfdsat(v1,v2,muR,gR,freR,fcoR,muG,gG,freG,fcoG,yo,dc);
%  (oppure:     [U1,W,U0]=mdfdsat(v1,v2,nuR,deR,nuG,deG,yo,dc);     )
%  visualizza e fornisce la soluz., applicando le f.d. a duplice ingresso.
%  La i-esima colonna di U0 corrisponde a U1(i). In mancanza, dc=0, yo=0.
%* L'orientamento e' indicato da un asterisco nel punto finale (se in fig.).

% G.O.Guardabassi, 3 gennaio 1998, 21 marzo 2000.

N=300;		% Aumentando questo parametro si aumenta la risoluzione
		% ovvero la precisione nel calcolo delle intersezioni;
		% il prezzo e' un forte appesantimento computazionale
		% nel caso di FD a duplice ingresso.
		% Se il punto d'intersezione evidenziato dai grafici e'
		% molto lontano dall'origine (rispetto a -1) e N non e'
		% sufficientemente elevato, mdfdsat puo' mancare di cal-
		% colare i valori di U1,W,U0. Il rimedio e' ovvio.
FD=1; if nargout==3, FD=2; end
					
if nargin < 6 | nargin == 9 %%%%%%%%%%%%%%%%%%%%%%%%%%%   Dati incongrui
	disp(' ')
	disp('   ATTENZIONE!')
	disp('   Ci sono troppo pochi argomenti.') 
	disp('                          Esecuzione sospesa.')
elseif nargin > 12 
	disp(' ')
	disp('   ATTENZIONE!')
	disp('   Ci sono troppi argomenti.') 
	disp('                          Esecuzione sospesa.')
end
%                    ***  ***  ***
if nargin < 9
	nuR=muR; deR=gR; nuG=freR; deG=fcoR;
	if deR(length(deR))==0 
		R0=inf; 
	else 
		R0=nuR(length(nuR))/deR(length(deR));
	end
	if deG(length(deG))==0 
		G0=inf; 
	else 
		G0=nuG(length(nuG))/deG(length(deG));
	end
	num=conv(nuR,nuG);
	den=conv(deR,deG);
	waz=abs(roots(num));
	wap=abs(roots(den));
	iz=find(waz>0);ip=find(wap>0);
	wa=[waz(iz);wap(ip)];		    
	wmin=min(wa)/3;wmax=max(wa)*10; 
	if nargin==6, yo=0; dc=0; end
	if nargin==7, yo=muG; dc=0; end
	if nargin==8,yo=muG; dc=gG; end
     	d1=floor(log10(wmin));
	d2=ceil(log10(wmax));
	w=logspace(d1,d2,N);
else
   [nuR,deR]=f2nd(muR,gR,freR,fcoR);
   [nuG,deG]=f2nd(muG,gG,freG,fcoG);   
	num=conv(nuR,nuG);
	den=conv(deR,deG);

	if gR==0, R0=muR;
	elseif gR>0, R0=inf;
	else, 	R0=0;
	end
	if gG==0, G0=muG;
	elseif gG>0, G0=inf;
	else, 	G0=0;
	end

	if nargin < 12, dc=0; end
	if nargin < 11, yo=0; end
   
   fre=[freR freG];
   fco=[fcoR fcoG];
   wangol=[];wnatur=[];
   if isempty(fre)==0,wangol=ones(1,length(fre(1,:)))./fre(1,:);end
   if isempty(fco)==0,wnatur=fco(1,:);end
   wx=abs([wangol wnatur]);
   
   wmin=min(wx)/3; wmax=max(wx)*10;
	d1=floor(log10(wmin));
	d2=ceil(log10(wmax));
	w=logspace(d1-1,d2+1,N);
end

[re,im]=nyquist(num,den,w); 	
xmin=min(0,min(re)); xmax=max(0,max(re)); 
ymin=min(0,min(im)); ymax=max(0,max(im));
LLx=xmax-xmin; LLy=ymax-ymin;
a1=(xmax+xmin)/2-.55*LLx; a2=a1+1.1*LLx;
a3=(ymax+ymin)/2-.55*LLy; a4=a3+1.1*LLy;

dLL=1.1*(LLx-LLy)/2;
if LLx > LLy
	a3=a3-dLL; a4=a4+dLL;
else
	a1=a1+dLL; a2=a2-dLL;
end

a=[a1 a2 a3 a4]; ex=[a1 a2]; ey=[a3 a4]; ze=[0 0];

pcx=[a1 -1]; pcy=ze;

clc
disp(' ')
disp('  *************************** ATTENZIONE **************************')
disp('  * Seguiranno due rappresentazioni, a diversa risoluzione, del   *')
disp('  * diagramma polare di R(jw)*G(jw) e del luogo dei punti critici.*')
disp('  * Per vedere la prima,  o per passare alla successiva,          *')
disp('  *                                              BATTERE UN TASTO.*')
disp('  *****************************************************************')
disp(' ')
if FD==2
 disp(' ')
 disp('  ****************************** NOTA *****************************')
 disp('  * Il ricorso alle funzioni descrittive a duplice ingresso puo''  *')
 disp('  * richiedere tempi di elaborazione relativ. lunghi (da qualche  *')
 disp('  * secondo a qualche minuto, nei casi peggiori).                 *')
 disp('  * A volte, occorre quindi attendere i risultati con  PAZIENZA.  *') 
 disp('  *****************************************************************')
 disp(' ')
end
pause;

ii=find((re<=0)&(im>=-LLy/10)&(im<=LLy/10));
m=length(ii);

if m==0
  disp(' ')
  disp('*************************** ATTENZIONE ***************************')
  disp('* Il diagramma polare di  R(jw)*G(jw)  non interseca il semiasse *')
  disp('* reale negativo.                                                *')
  disp('* Pertanto, la seconda rappresentazione viene omessa.            *')
  disp('*                                              BATTERE UN TASTO. *')
  disp('******************************************************************')
  disp(' ')
  pause
  E=[];
  W=[];
  Em=[];
end

figure(1)
    plot(re,im,'-k',ex,ze,'-k',ze,ey,'-k',re(N),im(N),'*g',pcx,pcy,'-r',...
    pcx(1),pcy(1),'*g'),axis(a);axis('square');grid,xlabel('Re'),ylabel('Im')
 	 title('Diagramma polare della r.i.f. di RG (in nero) e l.p.c. (in rosso)')
 	 pause;axis;

if m==0, return, end

xminr=min(re(ii)); xmaxr=max(re(ii)); 
LLxr=xmaxr-xminr; 
a1r=(xmaxr+xminr)/2-.55*LLxr; a2r=a1r+1.1*LLxr;
a3r=-.12*LLy; a4r=.12*LLy;
dLLr=1.1*(LLxr-.24*LLy)/2;
if LLxr > .24*LLy
a3r=a3r-dLLr; a4r=a4r+dLLr;
else
	a1r=a1r+dLLr; a2r=a2r-dLLr;
end
ar=[a1r a2r a3r a4r];exr=[a1r a2r];eyr=[a3r a4r];

W=zeri(w(ii),im(ii));
aar=spline(w(ii),re(ii),W);
D=-ones(size(W))./aar;
nD=length(D);
U=linspace(min([eps -v1 v2]),N*(v2-v1)/2,N);
Dh=.5*(ff(-v1*ones(1,N)./U)+ff(v2*ones(1,N)./U));

E=[];
W1=[];
for i=1:nD
	Di=D(i)*ones(1,N);
	Ei=zeri(U,Dh-Di);
	E=[E Ei];
	W1=[W1 W(i)*ones(1,length(Ei))];
end

Em=zeros(size(E));
	
figure(2)
    plot(re(ii),im(ii),'-k',exr,ze,'-k',ze,eyr,'-k',[a1r -1],pcy,'-r',...
    re(ii(length(ii))),im(ii(length(ii))),'*g',a1r,0,'*g')
 	 axis(ar);axis('square');grid,xlabel('Re'),ylabel('Im')
 	 title('Diagramma polare della r.i.f. di RG (in nero) e l.p.c. (in rosso)')
    axis;

if FD==1, W=W1; return, end

%%%%%%%%%%%%%%%%%%%%%%%%%  FD a duplice ingresso

U0=etasat(U,v1,v2,R0,G0,yo,dc);
gamma=U0-.5*(v2+v1)*ones(size(U0));
delta=.5*(v2-v1)*ones(size(U0));
[p,q]=size(U0);
Uh=kron(ones(p,1),U);
Dh=.5*(ff((delta+gamma)./Uh)+ff((delta-gamma)./Uh));

E=[];
W2=[];
for i=1:nD
	Di=D(i)*ones(size(Dh));
	Ei=zeri(U,Dh-Di);
	E=[E Ei];
	W2=[W2 W(i)*ones(1,length(Ei))];
end
W=W2;
Em=etasat(E,v1,v2,R0,G0,yo,dc);

