function   [mu,g,fre,fco]=nd2f(num,den)

% Se nuG,deG sono vettori che rappresentano numeratore e denominatore di una
% funzione di trasferimento G(s), la funzione
%
% 		     [muG,gG,freG,fcoG]=nd2f(nuG,deG);
%
% calcola le variabili che rappresentano G(s) in forma fattorizzata c.d.t.:
% 
% muG,gG       sono il guadagno e il tipo di G(s)
% freG,fcoG    sono matrici desunte da G(s) come segue:	
%	       freG : 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	       fcoG : 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli)
 
% G.O.Guardabassi, febbraio 1998, 8 febbraio 2000.

[z,p,ro]=tf2zp(num,den);

in=find(z==0);
nzo=0;
if length(in)>0
   nzo=in(length(in));
end
z=z(nzo+1:length(z));

ip=find(p==0);
npo=0;
if length(ip)>0
   npo=ip(length(ip));
end
p=p(npo+1:length(p));
g=npo-nzo;

if nzo*npo>0
   disp(' ')
   warning('ATTENZIONE! Cancellazione nell''origine.')
   disp(' ')
end
   
mu=ro;
fre=[];
fco=[];

inz=find((abs(imag(z))<=.001*abs(real(z)))&(abs(z)<inf));
for i=1:length(inz)
    	zz=real(z(inz(i)));
	mu=-mu*zz;
	fre=[fre [-1/zz; 1]];
end

inp=find((abs(imag(p))<=.001*abs(real(p)))&(abs(p)<inf));
for i=1:length(inp)
    	pp=real(p(inp(i)));
	mu=-mu/pp;
	fre=[fre [-1/pp; -1]];
end

in=find(imag(z)>.001*abs(real(z)));
for i=1:length(in)
    tso=conv([1 -z(in(i))],[1 -conj(z(in(i)))]);
    mu=mu*tso(3);
    fco=[fco [sqrt(tso(3)); .5*tso(2)/sqrt(tso(3)); 1]];
end

in=find(imag(p)>.001*abs(real(p)));
for i=1:length(in)
    tso=conv([1 -p(in(i))],[1 -conj(p(in(i)))]);
    mu=mu/tso(3);
    fco=[fco [sqrt(tso(3)); .5*tso(2)/sqrt(tso(3)); -1]];
end

ic=[];
[nr,nc]=size(fre);
if nc==1,ic=[1];end 
for k=2:nc
	if abs((fre(1,k)-fre(1,k-1))/fre(1,k)) < .001
		fre(2,k)=fre(2,k)+fre(2,k-1);
	else
		ic=[ic k-1];
	end
	if k==nc, ic=[ic k]; end
end
fre=fre(:,ic);

ic=[];
[nr,nc]=size(fco);
if nc==1,ic=[1];end
for k=2:nc
	eps1=abs((fco(1,k)-fco(1,k-1))/fco(1,k));
	eps2=abs(fco(2,k)-fco(2,k-1));
	if (eps1 < .001) & (eps2 < .001)
		fco(3,k)=fco(3,k)+fco(3,k-1);
	else
		ic=[ic k-1];
	end
	if k==nc, ic=[ic k]; end
end
fco=fco(:,ic);
