function [re,im,w]=nyq(mu,g,fre,fco,wmin,wmax,wp)
%* Per un'assegnata funzione di trasferimento L(s), nyq fornisce, in 2 figu-
%  re a diversa risoluzione, tratti (coniugati) del diagramma di Nyquist.
%* Se nuL e deL sono i vettori "numeratore" e "denominatore" di L(s), 
% 			  nyq(nuL,deL,w1,w2,wp);
%  visualizza la parte di diagramma corrispondente alla piu' piccola banda 
%  (in decadi) contenente [w1,w2] e indica con un '+' i punti corrispondenti
%  alle pulsazioni contenute nel vettore wp (che puo' essere omesso). 
%  Se omessi (insieme a wp), w1 e w2 vengono valutati da nyq.
%* Se: muL,gL  sono il guadagno e il tipo di L(s)
%    freL,fcoL sono matrici desunte da L(s) come segue:	
%	       freL : 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	       fcoL : 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli)
%  allora
%		      nyq(muL,gL,freL,fcoL,w1,w2,wp);
%  visualizza la parte di diagramma corrispondente alla piu' piccola banda 
%  (in decadi) contenente [w1,w2] e indica con un '+' i punti corrisponden-
%  ti a wp. Se omessi (insieme a wp), w1 e w2 sono valutati da nyq.
%* L'orientamento di un diagramma e' indicato da un asterisco (verde) nel 
%  punto finale, corrispondente alla massima pulsazione considerata.

% nyq usa: nd2f, f2nd, as

% G.O.Guardabassi, 1 gennaio 1997, 6 febbraio 2000.

N=600;

teta=linspace(0,2*pi,200);
xc=sin(teta);yc=cos(teta);

indnd=0;
if length(g) > 1
	num=mu; den=g;
	if nargin==5, wp=wmin; inwp=1; else, wp=[]; inwp=0; end
	inwmm=0;
	if nargin>=3, wmin=fre; end
	if nargin>=4, wmax=fco; inwmm=1; end
	[mu,g,fre,fco]=nd2f(num,den);
	indnd=1;
else
	inwp=1; inwmm=1;
	if nargin < 7, wp=[];inwp=0; end
	if nargin < 6, inwmm=0; end
end

if indnd==0, [num,den]=f2nd(mu,g,fre,fco); end

if inwmm==0, wmin=inf; wmax=inf; end
[mas,wmas,fas,wfas,w]=as(mu,g,fre,fco,wmin,wmax,N);
[m,f]=bode(num,den,w);
re=real(m.*exp(j*f*pi/180));
im=imag(m.*exp(j*f*pi/180));
if inwp==1
	iw=[];							
	for i=1:length(wp)					
		[mindw,imin]=min(abs(wp(i)*ones(1,N)-w));	
		iw=[iw imin(1)];				
	end							

else
	iw=[];							
end

xmin=min(re);xmax=max(re);ymin=min([im' -im']);ymax=-ymin;
LL=max(xmax-xmin,ymax-ymin);
a1=(xmax+xmin)/2-.55*LL;a2=a1+1.1*LL;
a3=-(a2-a1)/2;a4=-a3;
a=[a1 a2 a3 a4];ex=[a1 a2];ey=[a3 a4];ze=[0 0];

clc
disp(' ')
disp('  *************************** ATTENZIONE *************************')
disp('  * Seguiranno due rappresentazioni, a diversa risoluzione, del  *')
disp('  * diagramma di Nyquist.                                        *')
disp('  * Per vedere la prima,  o per passare alla successiva,         *')
disp('  *                                            BATTERE UN TASTO. *')
disp('  ****************************************************************')
disp(' ')
pause;

figure(1)
        plot(re,im,'-b',re,-im,':b',re(iw),im(iw),'+m',ex,ze,'-k',ze,ey,...
	'-k',re(N),im(N),'*g'),axis(a);axis('square');grid
	title('Diagramma di Nyquist'),xlabel('Re'),ylabel('Im'),pause;axis;

a=[-4 2 -3 3];
ex=[a(1) a(2)];ey=[a(3) a(4)];

figure(2)
		  plot(re,im,'-b',re,-im,':b',re(iw),im(iw),'+m',ex,ze,'-k',ze,ey,...
	'-k',re(N),im(N),'*g',xc,yc,'--w',-1,0,'or'),axis(a);axis('square');
	grid,title('Diagramma di Nyquist'),xlabel('Re'),ylabel('Im'),pause;
	axis;


