function  [m,f,w]=rif(mu,g,fre,fco,wmin,wmax,wp)

%* La funzione rif fornisce, in 5 figure, diverse visualizzazioni della 
%  risposta in frequenza associata ad una funzione di trasferimento G(s).
%* Se nuG e deG sono i vettori "numeratore" e "denominatore" di G(s), 
% 	([mG,fG,wG]=)      rif(nuG,deG,w1,w2,wp);
%  visualizza la risposta in frequenza sulla piu' piccola banda (in decadi)
%  contenente [w1,w2] e indica (ove serva) con un '+' i punti corrispondenti
%  alle pulsazioni contenute nel vettore wp (che puo' essere omesso).  Se
%  omessi (insieme a wp) w1 e w2 vengono valutati da rif.   
%* Se: muG,gG  sono il guadagno e il tipo di G(s)
%   freG,fcoG  sono matrici desunte da G(s) come segue:	
%	       freG : 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	       fcoG : 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli)
%  allora
%	([mG,fG,wG]=)   rif(muG,gG,freG,fcoG,w1,w2,wp);
%  visualizza la risposta in frequenza sulla piu' piccola banda (in decadi)
%  contenente [w1,w2] e indica (ove serva) con un '+' i punti corrispondenti
%  a wp. Se omessi (insieme a wp), w1 e w2 vengono valutati da rif.
%* mG=modulo (Nx1), fG=fase in gradi (Nx1), wG=pulsazione (1xN), N=500.
%* Nei diagrammi orientati, l'orientamento e' indicato da un asterisco nel 
%  punto finale, corrispondente alla massima pulsazione considerata.

% rif utilizza as, nd2f e f2nd.

% G.O.Guardabassi, 1 gennaio 1997, 20 marzo 1998, 6 febbraio 2000.

N=500;

indnd=0;
if length(g) > 1
	num=mu; den=g;
	if nargin==5, wp=wmin; inwp=1; else, wp=[]; inwp=0; end
	inwmm=0;
	if nargin>=3, wmin=fre; end
	if nargin>=4, wmax=fco; inwmm=1; end
	[mu,g,fre,fco]=nd2f(num,den);
	indnd=1;
else
	inwp=1; inwmm=1;
	if nargin < 7, wp=[];inwp=0; end
	if nargin < 6, inwmm=0; end
end

if indnd==0, [num,den]=f2nd(mu,g,fre,fco); end

if inwmm==0, wmin=inf; wmax=inf; end
[mas,wmas,fas,wfas,w]=as(mu,g,fre,fco,wmin,wmax,N);
[m,f]=bode(num,den,w);
mdb=20*log10(m);
re=real(m.*exp(j*f*pi/180));
im=imag(m.*exp(j*f*pi/180));
if inwp==1
	iw=[];							
	for i=1:length(wp)					
		[mindw,imin]=min(abs(wp(i)*ones(1,N)-w));	
		iw=[iw imin(1)];				
	end							

else
	iw=[];							
end	
xmin=min(re);xmax=max(re);ymin=min(im);ymax=max(im);
LL=1.2*max(xmax-xmin,ymax-ymin);
a1=1.2*min([xmin(1) 0]);a2=a1+LL;a3=1.2*min([ymin(1) 0]);a4=a3+LL;
a=[a1 a2 a3 a4];ex=[a1 a2];ey=[a3 a4];ze=[0 0];
dmas=max(mas)-min(mas);dfas=max(fas)-min(fas);
mMim=min(mas)-.1*dmas;mMam=max(mas)+.1*dmas;
fMim=min(fas)-.1*dfas;fMam=max(fas)+.1*dfas;

clc
disp(' ')
disp('  *************************** ATTENZIONE ***************************')
disp('  * Seguiranno varie rappresentazioni della risposta in frequenza. *')
disp('  * Per vedere la prima,  o per passare da ognuna alla successiva, *')
disp('  *                                              BATTERE UN TASTO. *')
disp('  ******************************************************************')
disp(' ')
pause;

figure(1)
    semilogx(w,mdb,':b',wmas,mas,'-m',wmas(1)*[1 1],[mMim mMam],'k')
    title('Diagrammi di Bode (esatto e asintotico): modulo')
    xlabel('pulsazione [rad/udt]'),ylabel('modulo [dB]'),grid,pause;
figure(2)
    semilogx(w,f,':b',wfas,fas,'-m',wfas(1)*[1 1],[fMim fMam],'k')
    title('Diagrammi di Bode (esatto e asintotico): argomento')
    xlabel('pulsazione [rad/udt]'),ylabel('argomento []'),grid,pause;
figure(3)
    subplot(211),semilogx(w,mdb),grid,ylabel('modulo [dB]')
	title('Diagrammi di Bode')
    subplot(212),semilogx(w,f),grid,ylabel('argomento []')
	xlabel('pulsazione [rad/udt]'),pause;subplot
figure(4)
    plot(ex,ze,'-k',ze,ey,'-k',re,im,'-b',re(N),im(N),'*g'),hold
    plot(re(iw),im(iw),'+m'),axis(a);axis('square');grid		
    title('Diagramma polare'),xlabel('Re'),ylabel('Im'),pause;axis;hold;
figure(5)
    plot(f,mdb,'-b',f(iw),mdb(iw),'+m',f(N),mdb(N),'*g'),grid
    title('Diagramma di Nichols'),xlabel('argomento []')
    ylabel('modulo [dB]'),hold;a=axis;plot(a([1 2]),[0 0],'-k'),hold

