%    simc(tf,In,nuP,deP,nuQ,deQ,nuR,deR,nuS,deS,nuT,deT,nuG,deG,nuH,deH);  
%
% simula, sull'intervallo [0, tf], un sistema di controllo monovariabile, a 
% tempo continuo, lineare e invariante nel tempo. Per una simulazione, oltre
% a tf, occorre fornire: la matrice In (che descrive l'andamento dei segnali 
% d'ingresso) e le funzioni di trasferimento degli elementi (blocchi) che 
% compongono il sistema.
%
% * Ogni riga della matrice In corrisponde a un ingresso (il riferimento c,
%   la 1a; il disturbo da, la 2a; e il disturbo dr, la 3a). Ogni ingresso ha 
%   la forma:         A sca(t) + B ram (t) + C par(t) + D sin(Wt)          .
%   Le ampiezze A, B, C, D e la pulsazione W sono indicate nella 1a, 2a etc.
%   fino alla 5a colonna di In, che quindi e' una matrice 3x5. Per "separare
%   gli effetti", se ci sono 2 ingressi non nulli il 2 viene fatto agire 
%   con un ritardo pari a tf/2; se ce ne sono 3, il 2 viene fatto agire con
%   un ritardo pari a tf/3 e il 3 con un ritardo pari a 2tf/3.
%
% * Ogni blocco del sistema e' contrassegnato da una lettera:  P=processo 
%   sotto controllo, Q=blocco fra il disturbo sulla linea d'andata e il nodo
%   somma all'uscita di P, T=trasduttore, S=blocco fra il disturbo sulla 
%   linea di retroazione e il nodo somma all'ingresso di T, R=controllore #1
%   (con ingresso y-y), G=convertitore del segnale di riferimento (da c a
%   y), H=controllore #2 (con ingresso y). Per ogni blocco, ad esempio P, 
%   la f.d.t. e' descritta da nuP (vettore dei coefficienti del numeratore 
%   ordinati per potenze decrescenti di s) e deP (quelli del denominatore).
%   In assenza; Q(s)=1, R(s)=1, S(s)=1, T(s)=1, G(s)=T(s), H(s)=0.

% G.O.Guardabassi, 5 gennaio 1997, 27 marzo 2000.

function [YY,t,uu]=...
         simc(tf,In,nuP,deP,nuQ,deQ,nuR,deR,nuS,deS,nuT,deT,nuG,deG,nuH,deH)

% YY(:,1)  :  c
% YY(:,2)  :  y
% YY(:,3)  :  u
% uu(:,1)  :  c
% uu(:,2)  :  da
% uu(:,3)  :  dr

clc
disp(' ')
disp(' *  USCITE:')
disp('    Variabile controllata:  c (uscita di P)')
disp('    Errore:  presunto e^ = c - y/muT,  effettivo e = c - c')
disp('    Variabile di controllo:  u (somma delle uscite di R e H)')
disp(' ')
disp(' ')
disp(' *  Per vedere la prima figura (e per passare alle successive)')
disp(' ')
disp(' 						     BATTERE UN TASTO')
disp(' ')
pause

N=600;   		    		    % Dev'essere un multiplo di 6.
t=linspace(0,tf,N);
uu1=In(1,1)*ones(1,N)+In(1,2)*t+In(1,3)*(t.^2)/2+In(1,4)*sin(In(1,5)*t);
uu2=In(2,1)*ones(1,N)+In(2,2)*t+In(2,3)*(t.^2)/2+In(2,4)*sin(In(2,5)*t);
uu3=In(3,1)*ones(1,N)+In(3,2)*t+In(3,3)*(t.^2)/2+In(3,4)*sin(In(3,5)*t);
ki=sign(sum(abs(In(:,1:4)')));
Np=N/sum(ki);
if sum(ki)==2
    if (ki(1)==0)|(ki(2)==0)
	uu3=[zeros(1,Np) uu3]; uu3=uu3(1:N);
    else
	uu2=[zeros(1,Np) uu2]; uu2=uu2(1:N);
    end 
end
if sum(ki)==3
    uu2=[zeros(1,Np) uu2]; uu2=uu2(1:N);
    uu3=[zeros(1,2*Np) uu3]; uu3=uu3(1:N);
end
uu=[uu1' uu2' uu3'];

if nargin < 16
    nuH=0;deH=1;
    if nargin < 14
    	if nargin < 12
	    nuT=1;deT=1;
	    if nargin < 10
		nuS=1;deS=1;
		if nargin < 8
		    nuR=1;deR=1;
		    if nargin < 6
			nuQ=1;deQ=1;
		    end
		end
	    end
	end
	nuG=nuT;deG=deT;
   end
end


[aP,bP,cP,dP]=tf2ss(nuP,deP);
[aQ,bQ,cQ,dQ]=tf2ss(nuQ,deQ);
[aR,bR,cR,dR]=tf2ss(nuR,deR);
[aS,bS,cS,dS]=tf2ss(nuS,deS);
[aT,bT,cT,dT]=tf2ss(nuT,deT);
[aG,bG,cG,dG]=tf2ss(nuG,deG);
[aH,bH,cH,dH]=tf2ss(nuH,deH);

if abs(dR*dP*dT)+abs(dH*dP*dT) > 0
    disp('*')
    disp('*   ATTENZIONE: Anello algebrico! Simulazione interrotta.')
    disp('*')
    return
end

nP=length(bP);nQ=length(bQ);nR=length(bR);nS=length(bS);nT=length(bT);
nG=length(bG);nH=length(bH);

if nP==0,bP=zeros(0,1);cP=zeros(1,0);end
if nQ==0,bQ=zeros(0,1);cQ=zeros(1,0);end
if nR==0,bR=zeros(0,1);cR=zeros(1,0);end
if nS==0,bS=zeros(0,1);cS=zeros(1,0);end
if nT==0,bT=zeros(0,1);cT=zeros(1,0);end
if nG==0,bG=zeros(0,1);cG=zeros(1,0);end
if nH==0,bH=zeros(0,1);cH=zeros(1,0);end

AA=[ aP-bP*(dR+dH)*dT*cP -bP*(dR+dH)*dT*cQ    bP*cR     -bP*(dR+dH)*dT*cS  -bP*(dR+dH)*cT      bP*dR*cG        -bP*cH    ;
        zeros(nQ,nP)             aQ        zeros(nQ,nR)    zeros(nQ,nS)     zeros(nQ,nT)     zeros(nQ,nG)   zeros(nQ,nH) ;
         -bR*dT*cP           -bR*dT*cQ    aR-bR*dT*dP*cR    -bR*dT*cS          -bR*cT           bR*cG       bR*dT*dP*cH  ;
        zeros(nS,nP)        zeros(nS,nQ)   zeros(nS,nR)         aS          zeros(nS,nT)     zeros(nS,nG)   zeros(nS,nH) ;
           bT*cP               bT*cQ         bT*dP*cR         bT*cS      aT-bT*dP*(dR+dH)*cT bT*dP*dR*cG    -bT*dP*cH    ;
        zeros(nG,nP)        zeros(nG,nQ)   zeros(nG,nR)    zeros(nG,nS)     zeros(nG,nT)          aG        zeros(nG,nH) ;
          bH*dT*cP            bH*dT*cQ     bH*dT*dP*cR       bH*dT*cS           bH*cT      bH*dT*dP*dR*cG aH-bH*dT*dP*cH];

CC=[ cP              cQ        dP*cR    zeros(1,nS)  -dP*(dR+dH)*cT    dP*dR*cG     -dP*cH   ;
   dT*cP            dT*cQ     dT*dP*cR     dT*cS            cT       dT*dP*dR*cG  -dT*dP*cH  ;
-(dR+dH)*dT*cP -(dR+dH)*dT*cQ    cR   -(dR+dH)*dT*cS   -(dR+dH)*cT      dR*cG        -cH    ];
      
BB=[ bP*dR*dG     zeros(nP,1) -bP*(dR+dH)*dT*dS;
    zeros(nQ,1)        bQ        zeros(nQ,1)   ;
       bR*dG       -bR*dT*dQ      -bR*dT*dS    ;
    zeros(nS,1)   zeros(nS,1)        bS        ;
    bT*dP*dR*dG      bT*dQ          bT*dS      ;
         bG       zeros(nG,1)    zeros(nG,1)   ;
   bH*dT*dP*dR*dG   bH*dT*dQ      bH*dT*dS    ];

DD=[  dP*dR*dG        dQ                0       ;
         0           dT*dQ            dT*dS     ;
         dR*dG    -(dR+dH)*dT*dQ  -(dR+dH)*dT*dS ];

YY=lsim(AA,BB,CC,DD,uu,t);

if det(aT)==0
    ep=zeros(N,1);
else
    muT=dT; if nT>=1, muT=dT-cT*inv(aT)*bT; end
    ep=uu1'-YY(:,2)/muT;
end
ee=uu1'-YY(:,1);

figure(1)
    subplot(221)
	plot(t,uu1,':r',t,YY(:,1),'-k',0,1.1*In(1,1),'-')
	ylabel('(---) c ,  () c')
	title('Riferimento e variabile controllata')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold
    subplot(222)
	plot(t,YY(:,3),'-k',[0 0],1.1*[max(YY(:,3)) min(YY(:,3))],'-k')
	ylabel('u'),title('Variabile di controllo')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold
    subplot(223)
    	plot(t,uu2,'-k',t,uu3,':m',0,1.1*max(In(2:3,1)),'-')
    	ylabel('() da ,  (---) dr'),xlabel('t'),title('Disturbi')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold
    subplot(224)
    	plot(t,ep,'-k',t,ee,':m',[0 0],1.1*[max([ep' ee']) min([ep' ee'])],'-k')
	xlabel('t'),ylabel('() e^ ,  (---) e')
	title('Errore presunto ed errore effettivo')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold,pause;
    subplot

figure(2)
    subplot(211)
	plot(t,uu1,':r',t,YY(:,1),'-k',0,1.1*In(1,1),'-')
	ylabel('(---) c ,  () c')
	title('Riferimento e variabile controllata')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold
if sum(max(abs(In(2:3,1:4))))==0
    subplot(212)
	plot(t,YY(:,3),'-k',[0 0],1.1*[max(YY(:,3)) min(YY(:,3))],'-k')
	xlabel('t'),ylabel('u'),title('Variabile di controllo')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold,pause;
    subplot
else
    subplot(212)
    	plot(t,uu2,'-k',t,uu3,':m',0,1.1*max(In(2:3,1)),'-')
    	ylabel('() da ,  (---) dr'),xlabel('t'),title('Disturbi')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold,pause;
    subplot

    figure(4)
	plot(t,YY(:,3),'-k',[0 0],1.1*[max(YY(:,3)) min(YY(:,3))],'-k')
	xlabel('t'),ylabel('u'),title('Variabile di controllo')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold,pause;
end

figure(3)
    	plot(t,ep,'-k',t,ee,':m',[0 0],1.1*[max([ep' ee']) min([ep' ee'])],'-k')
	xlabel('t'),ylabel('() presunto ,  (---) effettivo')
	title('Errore presunto ed errore effettivo')
    	v=axis;v(2)=tf;axis(v);hold,plot([0 v(2)],[0 0],'-k'),hold


