function [e,u,y,t]=simmbm(T,M,B,mu,g,fre,fco,yo,uo,deltauo)

% La funzione   [e,u,y,t]=simmbm(tf,M,B,nuG,deG,yo,uo,dc);
% o             [e,u,y,t]=simmbm(tf,M,B,muG,gG,freG,fcoG,yo,uo,dc);
% simula, su [0, tf], il sistema di controllo mostrato in figura. Le co-
% stanti yo,uo e dc (disturbo di carico) sono le ampiezze di tre scalini. 
% Il disturbo dc interviene a meta' dell'intervallo [0, tf], mentre yo e uo 
% sono presenti fin dall'inizio. In mancanza: yo=0, uo=0 e dc=0.
%
% * nuG,deG     sono il numeratore e il denominatore di G(s);
%		     (il grado del denominatore di G(s) dev'essere > 0)
% * muG,gG      sono il guadagno e il tipo di G(s)
%   freG,fcoG   sono matrici desunte da G(s) come segue:	
%	 freG : 1a riga, costanti di tempo reali (al num. o al den.)
%	        2a riga, corrispondenti molteplicita' (negative se al den.) 
%        fcoG : 1a riga, puls. nat. di coppie di zeri (poli) compl. coniug.
%      	        2a riga, corrispondenti fattori di smorzamento
%	       	3a riga, corrispondenti molteplicita' (negative se poli)
%                                                    
%                                    |uo+dc*sca(t-tf/2)
%                      ========      |      --------
%          yo +   e  ||        ||  + |+ u  |        |   y
%          ---->O--->||  MB/2  ||--->O---->|  G(s)  |----->
%              -|    ||        ||          |        |   |
%               |      ========             --------    |
%                ---------------------------------------

% simmbm usa f2nd

% G.O. Guardabassi, 24 dicembre 1997


if length(g)==1
    [nuG,deG]=f2nd(mu,g,fre,fco);
    if nargin < 10, deltauo=0; end
    if nargin < 9, uo=0; end
    if nargin < 8, yo=0; end
else
    if nargin==8, deltauo=yo; uo=fco; yo=fre;
    elseif nargin==7, deltauo=0; uo=fco; yo=fre;
    elseif nargin==6, deltauo=0; uo=0; yo=fre;
    elseif nargin==5, deltauo=0; uo=0; yo=0;
    elseif nargin < 5,
	disp(' ')
	disp(' I dati sono insufficienti.    ESECUZIONE SOSPESA!')
	disp(' ')
    end 
    nuG=mu;
    deG=g;
end

N=300;
t=linspace(0,T,2*N);
duo=[zeros(1,N) deltauo*ones(1,N)]';

[a,b,c,d]=tf2ss(nuG,deG);
if d ~= 0
	disp(' ')
	disp('Il sistema ha un anello algebrico. Simulazione interrotta')
	disp(' ')
	return
end

if yo >= -B/2
	S=1;
else
	S=-1;
end
u=(uo+S*M)*ones(2*N,1)+duo;

ii=0;

while ii <= 2*N
 	[y,x]=lsim(a,b,c,d,u,t);

	if S == 1
		Ind=find(y(ii+1:2*N)>yo+B/2);
	else
		Ind=find(y(ii+1:2*N)<yo-B/2);
	end

	if length(Ind) > 0
		ii=ii+Ind(1);
	else
		ii=2*N+1;
	end

	S=-S;
	if ii <= 2*N
		u=u+[zeros(1,ii-1) 2*S*M*ones(1,2*N-ii+1)]';
	end
end

e=yo*ones(2*N,1)-y;
ymax=max(y); ymin=min(y); Dy=ymax-ymin; ymed=(ymax+ymin)/2;
ay=[0 T ymed-.55*Dy ymed+.55*Dy];
ae=[0 T yo-ymed-.55*Dy yo-ymed+.55*Dy];
umax=max(u); umin=min(u); Du=umax-umin+10*eps; umed=(umax+umin)/2;
au=[0 T umed-.55*Du umed+.55*Du];
tuo=[0 T/2 T/2 T];
Uo=[uo uo uo+deltauo uo+deltauo];

clc
  disp(' ')
  disp('   *************************** ATTENZIONE **************************')
  disp('   * Seguiranno due figure con i transitori su [0,tf] delle varia- *')
  disp('   * bili y,u (Fig.1), e della variabile e (Fig.2).                *')
  disp('   * Per vedere la prima figura, o per passare alla successiva     *')
  disp('   *                                             BATTERE UN TASTO. *')
  disp('   *****************************************************************')
  disp(' ')
  pause

figure(1),close,figure(1)
    subplot(211),plot(t,y,'-',[0 T],[yo yo],'-.',[0 T],[0 0],'-')
    ylabel('y'),axis(ay);title('Esaminata la figura, BATTI UN TASTO!'),
    subplot(212),plot(t,u,'-',tuo,Uo,'-.',[0 T],[0 0],'-'),xlabel('t')
    ylabel('u'),axis(au);pause,subplot
figure(2),close,figure(2)
    plot(t,e,'-',[0 T],[0 0],'-'),xlabel('t'),ylabel('e'),axis(ae)