function     [y,u,v,t,xf]=simsat(tf,v1,v2,muR,gR,freR,fcoR,...
			         muG,gG,freG,fcoG,yo,dc,xo);

% La funzione       [y,u,v,t]=simsat(tf,v1,v2,nuR,deR,nuG,deG,yo,dc);
% o      [y,u,v,t]=simsat(tf,v1,v2,muR,gR,freR,fcoR,muG,gG,freG,fcoG,yo,dc);
% simula, su [0, tf], il sistema di controllo mostrato in figura. Le co-
% stanti yo e dc (disturbo di carico) sono le ampiezze di due scalini: il 
% disturbo dc interviene a meta' dell'intervallo [0, tf], mentre yo e' pre-
% sente fin dall'inizio. In mancanza, yo=0 e dc=0.
% * nuR,deR,nuG,deG sono il numeratore e il denominatore di R(s) e di G(s);
% * muR,gR,muG,gG   sono il guadagno e il tipo di R(s) e di G(s)
%   freR,fcoR,freG,fcoG  sono matrici desunte da R(s) e da G(s) come segue:	
%	  fre : 1a riga, costanti di tempo reali (al num. o al den.)
%	        2a riga, corrispondenti molteplicita' (negative se al den.) 
%	  fco : 1a riga, puls. nat. di coppie di zeri (poli) compl. coniug.
%       	2a riga, corrispondenti fattori di smorzamento
%	       	3a riga, corrispondenti molteplicita' (negative se poli)
%
%                                                    |dc*sca(t-tf/2)
%                --------       ==============       |     --------
%    yo  +      |        | u  ||              || v + |+   |        |   y
%    ----->O--->|  R(s)  |--->|| Sat(.,v1,v2) ||---->O--->|  G(s)  |----->   
%         -|    |        |    ||              ||          |        |   |     
%          |     --------       ==============             --------    |      
%          |                                                           |
%           -----------------------------------------------------------
%                                       | b ,  y >= b
%                          Sat(u;a,b) :=| u ,  a < y < b
%                                       | a ,  y <= a

% simsat usa f2nd, simsatg

% G.O. Guardabassi, 31 dicembre 1997

if nargin > 10
	[nuR,deR]=f2nd(muR,gR,freR,fcoR);
	[nuG,deG]=f2nd(muG,gG,freG,fcoG);
	nR=length(deR)-1;
	nG=length(deG)-1;
	n=nR+nG;
	if nargin < 14, xo=zeros(n,1); end
	if nargin < 13, dc=0; end
	if nargin < 12, yo=0; end
else
	nuR=muR;
	deR=gR;
	nuG=freR;
	deG=fcoR;
	nR=length(deR)-1;
	nG=length(deG)-1;
	n=nR+nG;
	if nargin==10, yo=muG; dc=gG; xo=freG; 
	elseif nargin==9, yo=muG; dc=gG; xo=zeros(n,1);
	elseif nargin==8, yo=muG; dc=0; xo=zeros(n,1);
	elseif nargin==7, yo=0; dc=0; xo=zeros(n,1);
	end
end

[AR,BR,CR,DR]=tf2ss(nuR,deR);
[AG,BG,CG,DG]=tf2ss(nuG,deG);

if nR==0,BR=zeros(0,1);CR=zeros(1,0);end
if nG==0,BG=zeros(0,1);CG=zeros(1,0);end

A=[ AG   zeros(nG,nR);
  -BR*CG      AR     ];
B=[  BG          BG      zeros(nG,1);
 zeros(nR,1) zeros(nR,1)     BR     ];
C=[-DR*CG     CR     ;
     CG   zeros(1,nR)];
D=[0   0  DR;
   0   0   0]; 
ua=[0 yo]';
ub=[dc yo]';

N=30;
[va,uya,ta,xfa]=simsatg(tf/2,v1,v2,A,B,C,D,xo,ua,N);
[vb,uyb,tb,xf]=simsatg(tf/2,v1,v2,A,B,C,D,xfa,ub,N);
v=[va;vb];
uy=[uya;uyb];
u=uy(:,1);
y=uy(:,2);
t=[ta tb+ones(1,10*N)*tf/2];

clc
disp(' ')
disp('  *************************** ATTENZIONE ***************************')
disp('  * Seguiranno due figure con gli andamenti di y o di u (e v).     *')
disp('  * Per vedere la prima figura, o per passare alla successiva,     *')
disp('  *                                              BATTERE UN TASTO. *')
disp('  ******************************************************************')
disp(' ')
pause;

figure(1),close,figure(1)
    plot([0 tf],[0 0],'-k',t,y,'-k',[0 tf],[yo yo],'--r')
    title('Esaminata la figura, BATTI UN TASTO!')
    xlabel('t'),ylabel('(---) yo    ,    () y'),pause;
figure(2),close,figure(2)
    plot(t,u,'-k',[0 tf],[0 0],'-k',t,v,':r')
    title('Esaminata la figura, BATTI UN TASTO!')
    xlabel('t'),ylabel('() u    ,    (---) v')
