function 	[v,yz,t,xf]=simsatg(tf,v1,v2,A,B,C,D,xo,u,N);

% La funzione   [v,yz,t]=simsatg(tf,v1,v2,A,B,C,D,xo,u);   simula su [0, tf]
% il sistema S=(A,B,C,D), con ingresso |v u'|', uscita |y z'|', u=costante e
% v=sat(y;v1,v2); xo e xf sono lo stato iniziale e lo stato finale di S. 
% Il "passo d'integrazione" e' tf/(10*N). In mancanza, u=0, xo=0, N=60.
%
%                                   |u
%            ============       ---------
%          ||            || v  |         |   y
%     O--->||sat(.,v1,v2)||--->|    S    |----->              |b , y >= b
%     |    ||            ||    |         |   |    sat(y;a,b):=|y , a < y < b
%     |      ============       ---------    |                |a , y <= a
%     |                             |z       |
%     |                                      |
%      --------------------------------------

% G.O. Guardabassi, 31 dicembre 1997

dimA=size(A);
dimB=size(B);
dimC=size(C);
lu=dimB(2)-1;
lz=dimC(1)-1;
if nargin < 10, N=60; end
t=linspace(0,tf,N*10);
if nargin < 9, u=zeros(lu,1); end
if nargin < 8, xo=zeros(dimA(1),1); end 

Bv=B(:,1);
Bu=B(:,2:lu+1);
Cy=C(1,:);
Cz=C(2:lz+1,:);
Dyv=D(1,1);
Dyu=D(1,2:lu+1);
Dzv=D(2:lz+1,1);
Dzu=D(2:lz+1,2:lu+1);
if Dyv ~= 0
	disp(' ')
	disp('ATTENZIONE: Anello algebrico. Simulazione interrotta!');
	disp(' ')
end

F=A+Bv*Cy;
G=Bu+Bv*Dyu;
Hy=Cy;
Hz=Cz+Dzv*Cy;
H=[Hy;Hz];
Ky=Dyu;
Kz=Dzu+Dzv*Dyu;
K=[Ky;Kz];

yoh=Hy*xo+Ky*u;
if (yoh >= v1)&(yoh <= v2)
	v=yoh; 
	s=0;
elseif yoh < v1
	v=v1;
	s=-1;
else
	v=v2;
	s=1;
end
yz=[yoh Hz*xo+Kz*u];

stop=0;
tN=t(1:N);
v1N=v1*ones(N,1);
v2N=v2*ones(N,1);
uN=kron(ones(N,1),u');

while stop==0
	k=length(v);
	if s==0
		[yzN,xN]=lsim(F,G,H,K,uN,tN,xo);
		yN=yzN(:,1);
		i1=find(yN<v1);
		i2=find(yN>v2);
		if length(i1)+length(i2)==0 
			xo=xN(N,:)'; 
			yz=[yz;yzN(2:N,:)];
			v=[v;yN(2:N)];
		else
			if length(i1)==0, i1(1)=N+1; end
			if length(i2)==0, i2(1)=N+1; end
			if i1(1) < i2(1) 
				xo=xN(i1(1),:)'; 
				yz=[yz;yzN(2:i1(1),:)];
				v=[v;yN(2:i1(1)-1);v1];
				s=-1;
			else
				xo=xN(i2(1),:)';
				yz=[yz;yzN(2:i2(1),:)];
				v=[v;yN(2:i2(1)-1);v2];				
				s=1;
			end
		end
	elseif s==-1
		[yzN,xN]=lsim(A,B,C,D,[v1N uN],tN,xo);
		yN=yzN(:,1);
		i=find(yN>v1);
		if length(i)==0 
			xo=xN(N,:)'; 
			yz=[yz;yzN(2:N,:)]; 
			v=[v;v1N(2:N)];
		else 
			xo=xN(i(1),:)'; 
			yz=[yz;yzN(2:i(1),:)];
			v=[v;v1N(2:i(1))];
			s=0;
		end
	else
		[yzN,xN]=lsim(A,B,C,D,[v2N uN],tN,xo);
		yN=yzN(:,1);
		i=find(yN<v2);
		if length(i)==0 
			xo=xN(N,:)'; 
			yz=[yz;yzN(2:N,:)]; 
			v=[v;v2N(2:N)];
		else 
			xo=xN(i(1),:)'; 
			yz=[yz;yzN(2:i(1),:)];
			v=[v;v2N(2:i(1))];
			s=0;
		end
	end
	[p,q]=size(yz);
	if p > 10*N, yz=yz(1:10*N,:); v=v(1:10*N); end
	if p >= 10*N, xf=xN(10*N-k,:)'; stop=1; end
end		


			






