% La funzione     [nuRtm,deRtm,nuRrp,deRrp]=tmrp(nuG,deG,T,tau);
% oppure	  [nuRtm,deRtm,nuRrp,deRrp]=tmrp(muG,gG,freG,fcoG,T,tau);
%
% calcola numeratore e denominatore del controllore in tempo minimo (nuRtm,
% deRtm) e del controllore a risposta piatta (nuRrp,deRrp) per un sistema 
% sotto controllo nel quale la funzione di trasferimento dall'uscita u del-
% l'organo di tenuta all'ingresso y del campionatore sia G(s)*exp(-tau*s).
%
% nuG,deG    numeratore e denominatore di G(s)
% muG,gG     guadagno e tipo di G(s)
% freG,fcoG  matrici desunte da G(s) come segue:
%	     freG : 1a riga, costanti di tempo reali (al num. o al den.)
%                   2a riga, corrispondenti molteplicita' (negative se al 
%	  	             denominatore) 
%	     fcoG : 1a riga, pulsazioni naturali relative a coppie di zeri
%			     o poli complessi coniugati
%       	    2a riga, corrispondenti fattori di smorzamento
% 	       	    3a riga, corrispondenti molteplicita' (negative se poli)
% T	     periodo di campionamento
% tau	     ritardo (rit. di elaborazione piu' rit. di processo)
%
% In mancanza, tau=0.

% tmrp usa f2nd, c2dd, tmrpd

% G.O.Guardabassi, febbraio 1998.

function	[nuRtm,deRtm,nuRrp,deRrp]=tmrp(muG,gG,freG,fcoG,T,tau)

if nargin > 4 
	[nuG,deG]=f2nd(muG,gG,freG,fcoG);
	if nargin < 6, tau=0; end
else
	nuG=muG;
	deG=gG;
	T=freG;
	if nargin== 4, tau=fcoG; end
	if nargin < 4, tau=0; end
end
[nuGd,deGd]=c2dd(nuG,deG,T,tau);
[nuRtm,deRtm,nuRrp,deRrp]=tmrpd(nuGd,deGd);
