% La funzione
%                       [nuR,deR]=tustin(T,nuRo,deRo);
% oppure 	  [nuR,deR]=tustin(T,muRo,gRo,freRo,fcoRo);
%
% calcola, con il metodo di Tustin, il numeratore nuR e il denominatore deR 
% della funzione di trasferimento a tempo discreto R(z) corrispondente a una
% funzione di trasferimento a tempo continuo R(s) con numeratore nuRo e de-
% nominatore deRo; T e' il periodo di campionamento.
% Alternativamente,
%  muRo,gRo    sono il guadagno e il tipo di R(s)
%  freRo,fcoRo sono matrici desunte da R(s) come segue:	
%	       freRo: 1a riga, costanti di tempo reali (al num. o al den.)
%	              2a riga, corrispondenti molteplicita' (negative se al 
%			       denominatore)
%	       fcoRo: 1a riga, pulsazioni naturali relative a coppie di zeri 
%			       o poli complessi coniugati
%	       	      2a riga, corrispondenti fattori di smorzamento
%	       	      3a riga, corrispondenti molteplicita' (negative se 
%			       poli).

% tustin chiama f2ndas

% G.O.Guardabassi, rev. 15 febbraio 1998.

function [nuR,deR]=tustin(T,muRo,gRo,freRo,fcoRo)         

if nargin < 5, fcoRo=[]; end
if nargin > 3, [nro,dro]=f2nd(muRo,gRo,freRo,fcoRo); end
if nargin== 3, nro=muRo; dro=gRo; end

f=2/T;
n=length(dro)-1;
fv=f.^(n:-1:0);
gr=n-length(nro)+1;
if gr>0
        nro=[zeros(1,gr) nro];
elseif gr<0
        disp('Sistema non causale')
        return
end

u=ones(1,n);
Q=poly(u);
for k=1:n
        u(k)=-1;
        Q=[   Q
           poly(u)];
end

deR=(dro.*fv)*Q;
nuR=((nro.*fv)*Q)/deR(1);
deR=deR/deR(1);

