function        [KpZN,TiZN,TdZN,Kpc,Tc]=zn(muG,gG,freG,fcoG,tau)

% La funzione   [KpZN,TiZN,TdZN,Kpc,Tc]=zn(nuG,deG,tau);
% oppure	       [KpZN,TiZN,TdZN,Kpc,Tc]=zn(muG,gG,freG,fcoG,tau);
%
% determina, con le regole di Ziegler e Nichols, i parametri di un PID per 
% un impianto avente funzione di trasferimento exp(-tau*s)*G(s), dove G(s)
% e' una funzione razionale di grado relativo positivo o nullo. 
%
% nuG,deG	 numeratore e denominatore di G
%
% muG,gG         guadagno e tipo di G
% freG,fcoG      matrici desunte da G come segue:
%	
%	 freG :  1a riga, costanti di tempo reali (al num. o al den.)
%	       	 2a riga, corrispondenti molteplicita' (negative se al den.)
%	 fcoG :  1a riga, pulsazioni naturali relative a coppie di zeri 
%			  o poli complessi coniugati
%	       	 2a riga, corrispondenti fattori di smorzamento
%	       	 3a riga, corrispondenti molteplicita' (negative se poli)
%
% tau		 ritardo. In mancanza, tau=0.	       
%
% KpZN,TiZN,TdZN valori trovati per Kp, Ti e Td.
% Kpc,Tc         valore critico del coeff. dell'azione prop. e periodo della   
%		 corrispondente oscillazione permanente.

% zn usa nd2f, as, f2nd

% G.O.Guardabassi, 23 aprile 1995, 15 marzo 1998

N=600;

if nargin == 2|nargin == 4, tau=0; end
if nargin == 3, tau=freG; end
if nargin <= 3,	[muG,gG,freG,fcoG]=nd2f(muG,gG); end

i1=[]; if isempty(freG)==0, i1=find(freG(1,:)<0); end
i2=[]; if isempty(fcoG)==0, i2=find(fcoG(2,:)<0); end
if length([i1 i2]) > 0
	disp(' ')
	disp('G e'' instabile. Impossibile applicare ZN')
	disp(' ')
	return
end

[mas,wmas,fas,wfas]=as(muG,gG,freG,fcoG);
[num,den]=f2nd(muG,gG,freG,fcoG);

d1=floor(log10(wmas(1)))-1;
d2=ceil(log10(wmas(length(wmas))))+1;
w=logspace(d1,d2,N);
[m,f]=bode(num,den,w);
f=f-w'*tau*180/pi;

i180=find(f<-180);
if length(i180)==0
	disp(' ')
	disp('Kc e'' illimitato. Non si puo'' applicare ZN')
	disp(' ')
	return
else
	w180=w(i180(1));
	Tc=2*pi/w180;
	Kpc=1/m(i180(1));
	KpZN=.6*Kpc;
	TiZN=Tc/2;
	TdZN=Tc/8;
end
